// This vertex shader is used to do per vertex diffuse and specular lighting
vs.1.1

// Compute the projection space vertex position
m4x4 oPos, v0, c0

// The first texture coordinate gets the texture coordinate of the mesh
mov oT0, v7

// Compute the normalized camera to vertex vector
sub r0, v0, c4
dp3 r0.w, r0, r0
rsq r0.w, r0.w
mul r0.xyz, r0.xyz, r0.w

// Compute the normalized light to vertex vector
sub r1, v0, c5
dp3 r1.w, r1, r1
rsq r1.w, r1.w
mul r1.xyz, r1.xyz, r1.w

// Compute the reflection vector using the formula r=v-n*2*<v,n>
dp3 r0.w, r0, v3
mul r0.w, r0.w, c7.x
mad r0, v3, r0.w, r0

// Compute the dot product between the reflection vector and the light differenc vector
dp3 r0.w, r1, -r0
// Make sure that the value isn't smaller than zero
max r0.w, c7.y, r0.w
// Take the specular factor (the dot product) to the power of 16 and pass the result as 
// specular color
mul r0.w, r0.w, r0.w
mul r0.w, r0.w, r0.w
mul r0.w, r0.w, r0.w
mul oD1.rgb, r0.w, r0.w

// Compute the range based attenuation factor through the following formula:
// A=LightRange / (d*16)
mul r0.w, r1.w, r1.w
mul r1.w, c5.w, c5.w
mul r1.w, r1.w, c7.w
rcp r1.w, r1.w
mul r0.w, r0.w, r1.w

// Compute the wave angel dependent lighting factor for the diffuse light
dp3 r1.w, v3, -r1
// Make sure that the value isn't smaller than zero
max r1.w, c7.y, r1.w
// Multiply the result by the attenuation factor
mul r0.w, r0.w, r1.w
// Multiply the diffuse lighting factor by the diffuse color of the material
mul r0.rgb, r0.w, c6
// Multiply the result by the light color
mul r0.rgb, r0.rgb, c10
// Multiply the ambient lighting color by the ambient color of the material and add 
// the diffuse color to get the final diffuse color
mov r1, c8
mad oD0.rgb, r1.rgb, c9.rgb, r0.rgb

// Pass the alpha component of the material color
mov oD0.a, c6.a
